@@ -50,6 +50,7 @@ typedef struct {
     double time_limit;
     long confl_limit;
 } Solver;
+typedef void (*sighandler_t)(int);
 
 static const char solver_create_docstring[] = \
 "Solver(verbose=0, time_limit=max_numeric_limits, confl_limit=max_numeric_limits, threads=1)\n\
@@ -362,28 +363,21 @@ static int _add_clauses_from_array(Solver *self, const size_t array_length, cons
     return 1;
 }
 
-static int _add_clauses_from_buffer_info(Solver *self, PyObject *buffer_info, const size_t itemsize)
+static int _add_clauses_from_buffer(Solver *self, Py_buffer *view)
 {
-    PyObject *py_array_length = PyTuple_GetItem(buffer_info, 1);
-    if (py_array_length == NULL) {
-        PyErr_SetString(PyExc_ValueError, "invalid clause array: could not get array length");
+    if (view->ndim != 1) {
+        PyErr_Format(PyExc_ValueError, "invalid clause array: expected 1-D array, got %d-D", view->ndim);
         return 0;
     }
-    long array_length = PyLong_AsLong(py_array_length);
-    if (array_length < 0) {
-        PyErr_SetString(PyExc_ValueError, "invalid clause array: could not get array length");
-        return 0;
-    }
-    PyObject *py_array_address = PyTuple_GetItem(buffer_info, 0);
-    if (py_array_address == NULL) {
-        PyErr_SetString(PyExc_ValueError, "invalid clause array: could not get array address");
-        return 0;
-    }
-    const void *array_address = PyLong_AsVoidPtr(py_array_address);
-    if (array_address == NULL) {
-        PyErr_SetString(PyExc_ValueError, "invalid clause array: could not get array address");
+    if (strcmp(view->format, "i") != 0 && strcmp(view->format, "l") != 0 && strcmp(view->format, "q") != 0) {
+        PyErr_Format(PyExc_ValueError, "invalid clause array: invalid format '%s'", view->format);
         return 0;
     }
+
+    void * array_address = view->buf;
+    size_t itemsize = view->itemsize;
+    size_t array_length = view->len / itemsize;
+
     if (itemsize == sizeof(int)) {
         return _add_clauses_from_array(self, array_length, (const int *) array_address);
     }
@@ -397,74 +391,14 @@ static int _add_clauses_from_buffer_info(Solver *self, PyObject *buffer_info, co
     return 0;
 }
 
-static int _check_array_typecode(PyObject *clauses)
-{
-    PyObject *py_typecode = PyObject_GetAttrString(clauses, "typecode");
-    if (py_typecode == NULL) {
-        PyErr_SetString(PyExc_ValueError, "invalid clause array: typecode is NULL");
-        return 0;
-    }
-
-    PyObject *typecode_bytes = PyUnicode_AsASCIIString(py_typecode);
-    Py_DECREF(py_typecode);
-    if (typecode_bytes == NULL) {
-        PyErr_SetString(PyExc_ValueError, "invalid clause array: could not get typecode bytes");
-        return 0;
-    }
-
-    const char *typecode_cstr = PyBytes_AsString(typecode_bytes);
-    if (typecode_cstr == NULL) {
-        Py_DECREF(typecode_bytes);
-        PyErr_SetString(PyExc_ValueError, "invalid clause array: could not get typecode cstring");
-        return 0;
-    }
-    const char typecode = typecode_cstr[0];
-    if (typecode == '\0' || typecode_cstr[1] != '\0') {
-        PyErr_Format(PyExc_ValueError, "invalid clause array: invalid typecode '%s'", typecode_cstr);
-        Py_DECREF(typecode_bytes);
-        return 0;
-    }
-    Py_DECREF(typecode_bytes);
-    if (typecode != 'i' && typecode != 'l' && typecode != 'q') {
-        PyErr_Format(PyExc_ValueError, "invalid clause array: invalid typecode '%c'", typecode);
-        return 0;
-    }
-    return 1;
-}
-
-static int add_clauses_array(Solver *self, PyObject *clauses)
-{
-    if (_check_array_typecode(clauses) == 0) {
-        return 0;
-    }
-    PyObject *py_itemsize = PyObject_GetAttrString(clauses, "itemsize");
-    if (py_itemsize == NULL) {
-        PyErr_SetString(PyExc_ValueError, "invalid clause array: itemsize is NULL");
-        return 0;
-    }
-    const long itemsize = PyLong_AsLong(py_itemsize);
-    Py_DECREF(py_itemsize);
-    if (itemsize < 0) {
-        PyErr_SetString(PyExc_ValueError, "invalid clause array: could not get itemsize");
-        return 0;
-    }
-    PyObject *buffer_info = PyObject_CallMethod(clauses, "buffer_info", NULL);
-    if (buffer_info == NULL) {
-        PyErr_SetString(PyExc_ValueError, "invalid clause array: buffer_info is NULL");
-        return 0;
-    }
-    int ret = _add_clauses_from_buffer_info(self, buffer_info, itemsize);
-    Py_DECREF(buffer_info);
-    return ret;
-}
-
 PyDoc_STRVAR(add_clauses_doc,
 "add_clauses(clauses)\n\
 Add iterable of clauses to the solver.\n\
 \n\
 :param clauses: List of clauses. Each clause contains literals (ints)\n\
-    Alternatively, this can be a flat array.array (typecode 'i', 'l', or 'q')\n\
-    of zero separated and terminated clauses of literals (ints).\n\
+    Alternatively, this can be a flat array.array or other contiguous\n\
+    buffer (format 'i', 'l', or 'q') of zero separated and terminated\n\
+    clauses of literals (ints).\n\
 :type clauses: <list> or <array.array>\n\
 :return: None\n\
 :rtype: <None>"
@@ -478,12 +412,16 @@ static PyObject* add_clauses(Solver *self, PyObject *args, PyObject *kwds)
         return NULL;
     }
 
-    if (
-        PyObject_HasAttr(clauses, PyUnicode_FromString("buffer_info")) &&
-        PyObject_HasAttr(clauses, PyUnicode_FromString("typecode")) &&
-        PyObject_HasAttr(clauses, PyUnicode_FromString("itemsize"))
-    ) {
-        int ret = add_clauses_array(self, clauses);
+    if (PyObject_CheckBuffer(clauses)) {
+        Py_buffer view;
+        memset(&view, 0, sizeof(view));
+        if (PyObject_GetBuffer(clauses, &view, PyBUF_CONTIG_RO | PyBUF_FORMAT) != 0) {
+            return NULL;
+        }
+
+        int ret = _add_clauses_from_buffer(self, &view);
+        PyBuffer_Release(&view);
+
         if (ret == 0 || PyErr_Occurred()) {
             return 0;
         }
@@ -702,6 +640,7 @@ Solve the system of equations that have been added with add_clause();\n\
 :rtype: <tuple <tuple>>"
 );
 
+
 static PyObject* solve(Solver *self, PyObject *args, PyObject *kwds)
 {
     PyObject* assumptions = NULL;
@@ -817,129 +756,6 @@ static PyObject* is_satisfiable(Solver *self)
     }
 }
 
-PyDoc_STRVAR(msolve_selected_doc,
-"msolve_selected(max_nr_of_solutions, var_selected, raw=True)\n\
-Find multiple solutions to your problem, the solver is ran in a loop and each\n\
-previous solution found will be banned.\n\
-\n\
-.. warning:: The loop will run as long as there are solutions.\n\
-    a maximum of loops must be set with 'max_nr_of_solutions' parameter\n\
-\n\
-.. note:: As it is highly suggested in the documentation of cryptominisat,\n\
-    the new clause (banned solutions) contains the variables that are \n\
-    \"important\" or \"main\" to your problem (i.e. \"var_selected\" argument).\n\
-    Variables that were only used to translate the original problem into CNF \n\
-    should not be added.\n\
-    This way, you will not get spurious solutions that don't differ in \n\
-    the main, important variables.\n\
-\n\
-:param max_nr_of_solutions: Maximum number of solutions before stop the search\n\
-:type max_nr_of_solutions: <int>\n\
-:param var_selected: Variables for which the solver must find different solutions\n\
-:type var_selected: <list>\n\
-:param raw: (Optional) Format of literals for each solution returned. \n\
-    If set to True, lists of literals will be returned;\n\
-    .. example:: [(1, -2, -3, -4, -5, -6, -7, -8, -9, 10,),]\n\
-    if set to False, tuples of booleans will be returned,\n\
-    with None at the first position.\n\
-    .. example:: [(None, True, False, True,),]\n\
-:type raw: <boolean>\n\
-:return: List of solutions (list of tuples of literals)\n\
-:rtype: <list <tuple>>"
-);
-
-static PyObject* msolve_selected(Solver *self, PyObject *args, PyObject *kwds)
-{
-    int max_nr_of_solutions;
-    int raw_solutions_activated = true;
-    PyObject *var_selected;
-
-    static char const* kwlist[] = {"max_nr_of_solutions", "var_selected", "raw", NULL};
-
-    // Use 'p' wildcard for the boolean on version 3.3+ of Python
-    if (!PyArg_ParseTupleAndKeywords(args, kwds, "iO|p", const_cast<char**>(kwlist),
-                                     &max_nr_of_solutions,
-                                     &var_selected,
-                                     &raw_solutions_activated)) {
-        return NULL;
-    }
-
-    std::vector<Lit> var_lits;
-    if (!parse_clause(self, var_selected, var_lits)) {
-        return 0;
-    }
-
-    PyObject *solutions = PyList_New(0);
-    if (solutions == NULL) {
-        PyErr_SetString(PyExc_SystemError, "failed to create a list");
-        return NULL;
-    }
-
-    int current_nr_of_solutions = 0;
-    lbool res = l_True;
-    PyObject* solution = NULL;
-    while((current_nr_of_solutions < max_nr_of_solutions) && (res == l_True)) {
-
-        Py_BEGIN_ALLOW_THREADS      /* release GIL */
-        res = self->cmsat->solve();
-        Py_END_ALLOW_THREADS
-
-        current_nr_of_solutions++;
-        if(res == l_True) {
-
-            // Memorize the solution
-            if (!raw_solutions_activated) {
-                // Solution in v5 format
-                solution = get_solution(self->cmsat);
-            } else {
-                // Solution in v2.9 format
-                solution = get_raw_solution(self->cmsat);
-            }
-
-            if (!solution) {
-                PyErr_SetString(PyExc_SystemError, "no solution");
-                Py_DECREF(solutions);
-                return NULL;
-            }
-            // Add solution
-            PyList_Append(solutions, solution);
-            Py_DECREF(solution);
-
-            // Prepare next statement
-            // Ban previous solution
-            if (current_nr_of_solutions < max_nr_of_solutions) {
-
-                std::vector<Lit> ban_solution;
-                const std::vector<lbool> model = self->cmsat->get_model();
-
-                // Iterate on var_selected (instead of iterate on all vars in solver)
-                for (unsigned long i = 0; i < var_lits.size(); i++) {
-                    assert(var_lits[i].var() < (uint32_t)self->cmsat->nVars());
-                    ban_solution.push_back(
-                        Lit(var_lits[i].var(), (model[var_lits[i].var()] == l_True) ? true : false)
-                    );
-                }
-
-                // Ban current solution for the next run
-                self->cmsat->add_clause(ban_solution);
-            }
-        } else if (res == l_False) {
-            // std::cout << "DEBUG :: Solver: No more solution" << std::endl;
-        } else if (res == l_Undef) {
-            Py_DECREF(solutions);
-            PyErr_SetString(PyExc_SystemError, "Nothing to do => sol undef");
-            return NULL;
-        } else {
-            // res can only be l_False, l_True, l_Undef
-            assert((res == l_False) || (res == l_True) || (res == l_Undef));
-            Py_DECREF(solutions);
-            return NULL;
-        }
-    }
-    // Return list of all solutions
-    return solutions;
-}
-
 PyDoc_STRVAR(get_conflict_doc,
 "get_conflict()\n\
 Returns the conflicts in the assumptions when the last call to solve(...)\n\
@@ -978,7 +794,6 @@ static PyMethodDef Solver_methods[] = {
     {"add_xor_clause",(PyCFunction) add_xor_clause,  METH_VARARGS | METH_KEYWORDS, "adds an XOR clause to the system"},
     {"nb_vars", (PyCFunction) nb_vars, METH_VARARGS | METH_KEYWORDS, nb_vars_doc},
     //{"nb_clauses", (PyCFunction) nb_clauses, METH_VARARGS | METH_KEYWORDS, "returns number of clauses"},
-    {"msolve_selected", (PyCFunction) msolve_selected, METH_VARARGS | METH_KEYWORDS, msolve_selected_doc},
     {"is_satisfiable", (PyCFunction) is_satisfiable, METH_VARARGS | METH_KEYWORDS, is_satisfiable_doc},
     {"get_conflict", (PyCFunction) get_conflict, METH_VARARGS | METH_KEYWORDS, get_conflict_doc},
 
@@ -1081,7 +896,11 @@ MODULE_INIT_FUNC(pycryptosat)
 
     // Add the version string so users know what version of CryptoMiniSat
     // they're using.
-    if (PyModule_AddStringConstant(m, "__version__", "${CMS_FULL_VERSION}") == -1) {
+    if (PyModule_AddStringConstant(m, "__version__", CMS_FULL_VERSION) == -1) {
+        Py_DECREF(m);
+        return NULL;
+    }
+    if (PyModule_AddStringConstant(m, "VERSION", CMS_FULL_VERSION) == -1) {
         Py_DECREF(m);
         return NULL;
     }
