@@ -24,7 +24,7 @@ set -e
 set -x
 set -o pipefail  # needed so  " | tee xyz " doesn't swallow the last command's error
 
-. ./setparams_ballofcrystal.sh
+source ./setparams_ballofcrystal.sh
 
 if [ "$1" == "--skip" ]; then
     echo "Will skip running CMS stats + FRAT"
@@ -47,7 +47,7 @@ if [ "$NEXT_OP" == "" ]; then
         echo "6 -- UTI-20-10p0.cnf-unz"
         echo "7 -- UCG-20-5p0.cnf"
 
-        read -p "Which CNF do you want to run? " myinput
+        read -r -p "Which CNF do you want to run? " myinput
         case $myinput in
             ["1"]* )
                 FNAME="velev-pipe-uns-1.0-9.cnf"
@@ -81,7 +81,7 @@ else
     initial="$(echo ${NEXT_OP} | head -c 1)"
     if [ "$1" == "-" ]; then
         echo "Cannot understand opion, there are no options"
-        exit -1
+        exit 255
     fi
     FNAME="${NEXT_OP}"
 fi
@@ -97,7 +97,7 @@ echo "--> with fixed number of data points  $FIXED"
 
 if [ "$FNAMEOUT" == "" ]; then
     echo "Error: FNAMEOUT is not set, it's empty. Exiting."
-    exit -1
+    exit 255
 fi
 
 
@@ -116,7 +116,7 @@ if [ "$SKIP" != "1" ]; then
     ########################
     # Build statistics-gathering CryptoMiniSat
     ########################
-    if [[ SANITIZE -eq 0 ]]; then
+    if [[ $SANITIZE -eq 0 ]]; then
         ./build_stats.sh
     else
         ./build_stats_sanitize.sh
@@ -128,29 +128,29 @@ if [ "$SKIP" != "1" ]; then
     ########################
     cd "$FNAME-dir"
     # for var, we need: --bva 0 --scc 0
-    $NOBUF ../cryptominisat5 --maxnummatrices 0 --presimp 1 -n1 --sqlitedbover 1 --cldatadumpratio "$DUMPRATIO" --cllockdatagen $CLLOCK --clid --sql 2 --sqlitedb "$FNAMEOUT.db-raw" --frat "$FNAMEOUT.frat" --zero-exit-status "../$FNAME" | tee cms-pred-run.out
+    $NOBUF ../cryptominisat5 --maxnummatrices 0 --presimp 1 -n1 --sqlitedbover 1 --cldatadumpratio "$DUMPRATIO" --cllockdatagen "$CLLOCK" --clid --sql 2 --sqlitedb "$FNAMEOUT.db-raw" --frat "$FNAMEOUT.frat" --zero-exit-status "../$FNAME" | tee cms-pred-run.out
     grep "c conflicts" cms-pred-run.out
 
     ########################
     # Run frat-rs
     ########################
     set +e
-    a=$(grep "s SATIS" cms-pred-run.out)
+    grep "s SATIS" cms-pred-run.out > /dev/null
     retval=$?
     set -e
     if [[ retval -eq 1 ]]; then
-        /usr/bin/time -v ../frat-rs elab "../$FNAME" "$FNAMEOUT.frat" -m -v
+        /usr/bin/time -v ../frat-rs elab "$FNAMEOUT.frat" "../$FNAME" -m -v
 
+        echo "If the following fails, you MUST compile frat-rs with 'ascii' feature through 'cargo build --features=ascii --release' !"
         set +e
-        /usr/bin/time -v ../frat-rs elab "$FNAMEOUT.frat" - tmp.lrat
+        /usr/bin/time -v ../frat-rs elab "$FNAMEOUT.frat"
         /usr/bin/time -v ../frat-rs refrat "$FNAMEOUT.frat.temp" correct
         set -e
-        #/usr/bin/time -v $NOBUF ../utils/drat-trim/drat-trim "../$FNAME" "$FNAMEOUT.drat" -o "$FNAMEOUT.usedCls" -i -O 4 -m | tee drat.out-newO4
 
 
     else
         echo "Not UNSAT!!!"
-        exit -1
+        exit 255
     fi
     echo "CMS+FRAT done now"
     )
@@ -173,7 +173,7 @@ rm -f check_quality.out
 rm -f clean_update.out
 rm -f fill_clauses.out
 
-../fix_up_frat.py correct $FNAMEOUT.db-raw | tee fill_used_clauses.out
+../fix_up_frat.py correct "$FNAMEOUT.db-raw" | tee fill_used_clauses.out
 cp "$FNAMEOUT.db-raw" "$FNAMEOUT.db"
 /usr/bin/time -v ../clean_update_data.py "$FNAMEOUT.db"  | tee clean_update_data.out
 ../check_data_quality.py --slow "$FNAMEOUT.db" | tee check_data_quality.out
@@ -184,7 +184,7 @@ cp "$FNAMEOUT.db" "$FNAMEOUT-min.db"
 ########################
 # Denormalize the data into a Pandas Table, label it and sample it
 ########################
-../cldata_gen_pandas.py "${FNAMEOUT}-min.db" --cut1 $cut1 --cut2 $cut2 --limit "$FIXED" ${EXTRA_GEN_PANDAS_OPTS} | tee cldata_gen_pandas.out
+../cldata_gen_pandas.py "${FNAMEOUT}-min.db" --cut1 "$cut1" --cut2 "$cut2" --limit "$FIXED" ${EXTRA_GEN_PANDAS_OPTS} | tee cldata_gen_pandas.out
 # ../vardata_gen_pandas.py "${FNAMEOUT}.db" --limit 1000
 
 
@@ -212,7 +212,7 @@ tables=("used_later" "used_later_anc")
 for tier in "${tiers[@]}"; do
     for table in "${tables[@]}"; do
         for regressor in "${regressors[@]}"; do
-            $NOBUF ../cldata_predict.py "${FNAMEOUT}-min.db-cldata-${table}-${tier}-cut1-$cut1-cut2-$cut2-limit-${FIXED}.dat" --tier ${tier} --table ${table} --features best_only --regressor $regressor --basedir "." --bestfeatfile $bestf | tee "cldata_predict_${tier}-${table}-${regressor}.out"
+            $NOBUF ../cldata_predict.py "${FNAMEOUT}-min.db-cldata-${table}-${tier}-cut1-$cut1-cut2-$cut2-limit-${FIXED}.dat" --tier "${tier}" --table "${table}" --features best_only --regressor "$regressor" --basedir "." --bestfeatfile "$bestf"| tee "cldata_predict_${tier}-${table}-${regressor}.out"
         done
     done
 done
