@@ -233,9 +233,9 @@ inline void CNF::updateWatch(
             }
         }
         if (!found) {
-            it->setBlockedLit(cl[2]);
+            it->setElimedLit(cl[2]);
         } else {
-            it->setBlockedLit(blocked_lit);
+            it->setElimedLit(blocked_lit);
         }
     }
 }
@@ -373,6 +373,7 @@ vector<uint32_t> CNF::build_outer_to_without_bva_map() const
     return ret;
 }
 
+// ONLY used during getting clauses for external uses. NOT sane
 vector<uint32_t> CNF::build_outer_to_without_bva_map_extended() const
 {
     assert(nVarsOutside() <= nVarsOuter());
@@ -811,7 +812,7 @@ vector<uint32_t> CNF::get_outside_lit_incidence()
     //Map to outer
     vector<uint32_t> inc_outer(nVarsOuter()*2, 0);
     for(uint32_t i = 0; i < inc.size(); i ++) {
-        Lit outer = map_inter_to_outer(Lit::toLit(i));
+        const Lit outer = map_inter_to_outer(Lit::toLit(i));
         inc_outer[outer.toInt()] = inc[i];
     }
 
@@ -984,6 +985,7 @@ void CNF::check_no_zero_ID_bins() const
     }
 }
 
+// This requires occurrence lists to be linked in
 bool CNF::zero_irred_cls(const CMSat::Lit lit) const
 {
     for(auto const& w: watches[lit]) {
