@@ -20,8 +20,7 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
 ***********************************************/
 
-#ifndef __CNF_H__
-#define __CNF_H__
+#pragma once
 
 #include <atomic>
 #include <limits>
@@ -42,6 +41,10 @@ THE SOFTWARE.
 #include <pseudoboolean.h>
 #endif
 
+#ifdef ARJUN_SERIALIZE
+#include <boost/serialization/vector.hpp>
+#endif
+
 namespace CMSat {
 
 class ClauseAllocator;
@@ -290,6 +293,14 @@ public:
     {
         return outerToInterMain[outer];
     }
+    void map_outer_to_inter(vector<uint32_t>& outer) const
+    {
+        for(auto& v: outer) v = outerToInterMain[v];
+    }
+    void map_inter_to_outer(vector<uint32_t>& inter) const
+    {
+        for(auto& v: inter) v = interToOuterMain[v];
+    }
     Lit map_outer_to_inter(const Lit outer) const
     {
         return Lit(outerToInterMain[outer.var()], outer.sign());
@@ -361,6 +372,14 @@ public:
     #ifdef USE_TBUDDY
     void free_bdds(vector<Xor>& xors);
     #endif
+
+#ifdef ARJUN_SERIALIZE
+    template<class T> void unserialize(T& ar);
+    template<class T> void serialize(T& ar) const;
+#endif
+    size_t get_num_long_cls() const;
+    size_t get_num_long_irred_cls() const;
+    size_t get_num_long_red_cls() const;
     void print_all_clauses() const;
     void print_watchlist_stats() const;
     bool zero_irred_cls(const Lit lit) const;
@@ -765,6 +784,33 @@ inline bool CNF::satisfied(const ClOffset& off) const
     return satisfied(*cl);
 }
 
+inline size_t CNF::get_num_long_irred_cls() const
+{
+    return longIrredCls.size();
 }
 
-#endif //__CNF_H__
+inline size_t CNF::get_num_long_red_cls() const
+{
+    return longRedCls.size();
+}
+
+inline size_t CNF::get_num_long_cls() const
+{
+    return longIrredCls.size() + longRedCls.size();
+}
+
+#ifdef ARJUN_SERIALIZE
+template<class T> void CNF::unserialize(T& ar)
+{
+    ar >> outer_to_with_bva_map;
+    ar >> num_bva_vars;
+}
+
+template<class T> void CNF::serialize(T& ar) const
+{
+    ar << outer_to_with_bva_map;
+    ar << num_bva_vars;
+}
+#endif
+
+}
