@@ -662,10 +662,6 @@ void Main::add_supported_options()
         , "Maximum matrix size (=num elements) that we should try to echelonize")
     ("forcepreservexors", po::value(&conf.force_preserve_xors)->default_value(conf.force_preserve_xors)
         , "Force preserving XORs when they have been found. Easier to make sure XORs are not lost through simplifiactions such as strenghtening")
-#ifdef USE_M4RI
-    ("m4ri", po::value(&conf.doM4RI)->default_value(conf.doM4RI)
-        , "Use M4RI")
-#endif
     //Not implemented yet
     //("mix", po::value(&conf.doMixXorAndGates)->default_value(conf.doMixXorAndGates)
     //    , "Mix XORs and OrGates for new truths")
@@ -928,11 +924,7 @@ void Main::check_options_correctness()
             << " DIMACS with XOR extension" << endl << endl;
 
             cout
-            << "cryptominisat5 [options] inputfile [frat-trim-file]" << endl << endl;
-
-            cout << "Preprocessor usage:" << endl
-            << "  cryptominisat5 --preproc 1 [options] inputfile simplified-cnf-file" << endl << endl
-            << "  cryptominisat5 --preproc 2 [options] solution-file" << endl;
+            << "cryptominisat5 [options] inputfile [frat-file]" << endl << endl;
 
             cout << help_options_complicated << endl;
             cout << "Normal run schedules:" << endl;
@@ -946,9 +938,7 @@ void Main::check_options_correctness()
         if (vm.count("help"))
         {
             cout
-            << "USAGE 1: " << argv[0] << " [options] inputfile [frat-trim-file]" << endl
-            << "USAGE 2: " << argv[0] << " --preproc 1 [options] inputfile simplified-cnf-file" << endl
-            << "USAGE 2: " << argv[0] << " --preproc 2 [options] solution-file" << endl
+            << "USAGE: " << argv[0] << " [options] inputfile [frat-trim-file]" << endl
 
             << " where input is "
             #ifndef USE_ZLIB
