@@ -1,6 +1,5 @@
 /*
-Copyright (c) 2010-2015 Mate Soos
-Copyright (c) Kuldeep S. Meel, Daniel J. Fremont
+Copyright (C) 2009-2020 Authors of CryptoMiniSat, see AUTHORS file
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -28,12 +27,13 @@ THE SOFTWARE.
 
 int main(int argc, char** argv)
 {
-    #if defined(__GNUC__) && defined(__linux__)
-    feenableexcept(FE_INVALID   |
-                   FE_DIVBYZERO |
-                   FE_OVERFLOW
-    );
-    #endif
+// --> HAD to be disabled due to python module
+//     #if defined(__GNUC__) && defined(__linux__)
+//     feenableexcept(FE_INVALID   |
+//                    FE_DIVBYZERO |
+//                    FE_OVERFLOW
+//     );
+//     #endif
 
     int ret = -1;
     try {
@@ -50,6 +50,9 @@ int main(int argc, char** argv)
     } catch (CMSat::TooLongClauseError& e) {
         std::cerr << "ERROR! Too long clause inserted" << std::endl;
         exit(-1);
+    } catch (const boost::program_options::error& ex) {
+        std::cerr << "Error: " << ex.what() << std::endl;
+        exit(-1);
     }
 
     return ret;
