@@ -20,9 +20,7 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
 ***********************************************/
 
-#ifndef SIMPLIFIER_H
-#define SIMPLIFIER_H
-
+#pragma once
 
 #include <map>
 #include <vector>
@@ -32,6 +30,9 @@ THE SOFTWARE.
 #include <map>
 #include <iomanip>
 #include <fstream>
+#ifdef ARJUN_SERIALIZE
+#include <boost/serialization/vector.hpp>
+#endif
 
 #include "clause.h"
 #include "solvertypes.h"
@@ -49,21 +50,19 @@ using std::vector;
 using std::map;
 using std::set;
 using std::pair;
-using std::priority_queue;
 
 class ClauseCleaner;
 class SolutionExtender;
 class Solver;
-class TopLevelGaussAbst;
 class SubsumeStrengthen;
 class BVA;
 class GateFinder;
 
-struct BlockedClauses {
-    BlockedClauses()
+struct ElimedClauses {
+    ElimedClauses()
     {}
 
-    explicit BlockedClauses(size_t _start, size_t _end) :
+    explicit ElimedClauses(size_t _start, size_t _end) :
         start(_start)
         , end(_end)
         , toRemove(false)
@@ -83,20 +82,28 @@ struct BlockedClauses {
         end = f.get_uint64_t();
     }
 
-    const Lit& at(const uint64_t at, const vector<Lit>& blkcls) const
+    const Lit& at(const uint64_t at, const vector<Lit>& eClsLits) const
     {
-        return blkcls[start+at];
+        return eClsLits[start+at];
     }
 
-    Lit& at(const uint64_t at, vector<Lit>& blkcls)
+    Lit& at(const uint64_t at, vector<Lit>& eClsLits)
     {
-        return blkcls[start+at];
+        return eClsLits[start+at];
     }
 
     uint64_t size() const {
         return end-start;
     }
 
+    template<class Archive>
+    void serialize(Archive& ar, const unsigned int /*version*/)
+    {
+        ar & start;
+        ar & end;
+        ar & toRemove;
+    }
+
     uint64_t start;
     uint64_t end;
     bool toRemove = false;
@@ -199,7 +206,7 @@ public:
 
     // definable vars
     vector<uint32_t> remove_definable_by_irreg_gate(const vector<uint32_t>& vars);
-    void find_equiv_subformula(vector<uint32_t>& empty_vars, vector<uint32_t>& sampl_vars, bool mirror_empty);
+    void clean_sampl_and_get_empties(vector<uint32_t>& empty_vars, vector<uint32_t>& sampl_vars);
     bool elim_var_by_str(uint32_t var, const vector<pair<ClOffset, ClOffset>>& cls);
     uint32_t add_cls_to_picosat_definable(const Lit wsLit);
     PicoSAT* picosat = NULL;
@@ -216,18 +223,17 @@ public:
     void save_on_var_memory();
     bool uneliminate(const uint32_t var);
     size_t mem_used() const;
-    size_t mem_used_xor() const;
     size_t mem_used_bva() const;
-    uint32_t dump_blocked_clauses(std::ostream* outfile) const;
-    bool get_blocked_clause_at(uint32_t& at,uint32_t& at2, vector<Lit>& out) const;
+    uint32_t dump_elimed_clauses(std::ostream* outfile) const;
+    bool get_elimed_clause_at(uint32_t& at,uint32_t& at2, vector<Lit>& out) const;
     void subs_with_resolvent_clauses();
     void fill_tocheck_seen(const vec<Watched>& ws, vector<uint32_t>& tocheck);
-    void delete_blocked_clauses(); ///< This REMOVES them, not blocks them. For Arjun.
     void delete_component_unconnected_to_assumps(); //for arjun
     void strengthen_dummy_with_bins(const bool avoid_redundant);
+    void blocked_clause_elim();
 
     //UnElimination
-    void print_blocked_clauses_reverse() const;
+    void print_elimed_clauses_reverse() const;
     void extend_model(SolutionExtender* extender);
     uint32_t get_num_elimed_vars() const
     {
@@ -262,14 +268,21 @@ public:
 
     const Stats& get_stats() const;
     const SubsumeStrengthen* get_sub_str() const;
+
+    //validity checking
     void check_elimed_vars_are_unassigned() const;
     void check_no_marked_clauses();
-    bool getAnythingHasBeenBlocked() const;
+    bool getAnythingHasBeenElimed() const;
+    void sanityCheckElimedVars() const;
+    void printOccur(const Lit lit) const;
+    void check_clauses_lits_ordered() const;
 
     /// Used ONLY for XOR, changes occur setup
     void sort_occurs_and_set_abst();
     vector<ClOffset> added_long_cl;
-    TouchList added_cl_to_var;
+    TouchList added_cl_to_var; // to perform forward-subsumption of clauses added as part of BVE
+                               // contains a list of literals that have been added as part of
+                               // the clauses that have been added as resolvents
     vector<uint32_t> n_occurs;
     TouchList removed_cl_with_var;
     vector<std::pair<Lit, Lit> > added_irred_bin;
@@ -289,10 +302,12 @@ public:
     //Ternary resolution. Should be private but testing needs it to be public
     bool ternary_res();
 
-    //validity checking
-    void sanityCheckElimedVars() const;
-    void printOccur(const Lit lit) const;
-    void check_clauses_lits_ordered() const;
+#ifdef ARJUN_SERIALIZE
+    template<class T>
+    void serialize_elimed_cls  (T& ar) const;
+    template<class T>
+    void unserialize_elimed_cls(T& ar);
+#endif
 
 private:
     friend class SubsumeStrengthen;
@@ -314,11 +329,7 @@ private:
     struct Tri {
         Lit lits[3];
         uint32_t size = 0;
-
-        Tri () :
-            size(0)
-        {}
-
+        Tri () : size(0) {}
         Tri(const Tri & other)
         {
             memcpy(lits, other.lits, sizeof(Lit)*3);
@@ -395,7 +406,7 @@ private:
     void set_limits();
 
     //Finish-up
-    void remove_by_frat_recently_blocked_clauses(size_t origBlockedSize);
+    void remove_by_frat_recently_elimed_clauses(size_t origElimedSize);
     void add_back_to_solver();
     bool check_varelim_when_adding_back_cl(const Clause* cl) const;
     void remove_all_longs_from_watches();
@@ -455,7 +466,7 @@ private:
 
     TouchList   elim_calc_need_update;
     vector<ClOffset> cl_to_free_later;
-    bool        maybe_eliminate(const uint32_t x);
+    bool        maybe_eliminate(const uint32_t var);
     bool        forward_subsume_irred(
         const Lit lit,
         cl_abst_type abs,
@@ -481,7 +492,7 @@ private:
         const vec<Watched>& gates,
         const vec<Watched>& full_set,
         vec<Watched>& output);
-    bool deal_with_added_long_and_bin(const bool verbose = true);
+    bool sub_str_with_added_long_and_bin(const bool verbose = true);
     vector<Lit> tmp_bin_cl;
     vec<Watched> gates_poss;
     vec<Watched> gates_negs;
@@ -493,7 +504,7 @@ private:
     void clean_from_red_or_removed(
         const vec<Watched>& in,
         vec<Watched>& out);
-    void        create_dummy_blocked_clause(const Lit lit);
+    void        create_dummy_elimed_clause(const Lit lit);
     vector<OccurClause> tmp_subs;
     bool        test_elim_and_fill_resolvents(uint32_t var);
     void        get_gate(Lit elim_lit, watch_subarray_const poss, watch_subarray_const negs);
@@ -635,22 +646,20 @@ private:
 
     /////////////////////
     //Helpers
-    friend class TopLevelGaussAbst;
     friend class GateFinder;
-    TopLevelGaussAbst *topLevelGauss;
     GateFinder *gateFinder;
 
     /////////////////////
-    //Blocked clause elimination
-    bool anythingHasBeenBlocked;
-    vector<Lit> blkcls;
-    vector<BlockedClauses> blockedClauses; ///<maps var(outer!!) to postion in blockedClauses
+    //Elimed clause elimination
+    bool anythingHasBeenElimed;
+    vector<Lit> eClsLits;
+    vector<ElimedClauses> elimedClauses; ///<maps var(outer!!) to postion in elimedClauses
     vector<uint32_t> blk_var_to_cls;
-    vector<int32_t> newly_blocked_cls_IDs; // temporary storage for newly blocked cls' IDs
-    bool blockedMapBuilt;
-    void buildBlockedMap();
-    void cleanBlockedClauses();
-    bool can_remove_blocked_clauses = false;
+    vector<int32_t> newly_elimed_cls_IDs; // temporary storage for newly elimed cls' IDs
+    bool elimedMapBuilt;
+    void buildElimedMap();
+    void cleanElimedClauses();
+    bool can_remove_elimed_clauses = false;
 
     ///Stats from this run
     Stats runStats;
@@ -664,18 +673,11 @@ inline const OccSimplifier::Stats& OccSimplifier::get_stats() const
     return globalStats;
 }
 
-inline bool OccSimplifier::getAnythingHasBeenBlocked() const
+inline bool OccSimplifier::getAnythingHasBeenElimed() const
 {
-    return anythingHasBeenBlocked;
+    return anythingHasBeenElimed;
 }
 
-/*inline std::ostream& operator<<(std::ostream& os, const BlockedClauses& bl)
-{
-    os << bl.lits << " to remove: " << bl.toRemove;
-
-    return os;
-}*/
-
 inline bool OccSimplifier::subsetReverse(const Clause& B) const
 {
     for (uint32_t i = 0; i != B.size(); i++) {
@@ -690,6 +692,20 @@ inline const SubsumeStrengthen* OccSimplifier::get_sub_str() const
     return sub_str;
 }
 
-} //end namespace
+#ifdef ARJUN_SERIALIZE
+template<class T>
+void OccSimplifier::unserialize_elimed_cls(T& ar)
+{
+    ar >> eClsLits;
+    ar >> elimedClauses;
+}
 
-#endif //SIMPLIFIER_H
+template<class T>
+void OccSimplifier::serialize_elimed_cls(T& ar) const
+{
+    ar << eClsLits;
+    ar << elimedClauses;
+}
+#endif
+
+} //end namespace
