@@ -20,37 +20,33 @@
 // OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE
 // OR OTHER DEALINGS IN THE SOFTWARE.
 
-#include "oracle.hpp"
+#include "oracle.h"
 
 #include <set>
 #include <queue>
+#include <iostream>
 
-#include "utils.hpp"
+#include "utils.h"
+using std::cout;
+using std::endl;
+using std::max;
+using std::min;
+using std::pair;
+using std::swap;
 
 namespace sspp {
 namespace oracle {
 namespace {
-const double EPS = 1e-150;
+constexpr const double EPS = 1e-150;
 } // namespace
 
 void Stats::Print() const {
-	cerr<<"Decisions/Propagations "<<decisions<<"/"<<mems<<endl;
-	cerr<<"Conflicts: "<<conflicts<<endl;
-	cerr<<"Learned clauses/bin/unit: "<<learned_clauses<<"/"<<learned_bin_clauses<<"/"<<learned_units<<endl;
-	cerr<<"Forgot clauses: "<<forgot_clauses<<endl;
-	cerr<<"Nontriv redu: "<<nontriv_redu<<endl;
-	cerr<<"Timers:"<<endl;
-	cerr<<"Setup "<<setup_timer.get()<<endl;
-	cerr<<"Solve "<<solve_timer.get()<<endl;
-	cerr<<"Propagation "<<prop_timer.get()<<endl;
-	cerr<<"Learn "<<learn_timer.get()<<endl;
-	cerr<<"Maint "<<maint_time.get()<<endl;
-	cerr<<"Cache "<<cache_timer.get()<<endl;
-	cerr<<"Restarts "<<restarts<<endl;
-}
-
-bool CInfo::Keep() const {
-	return glue <= 2 || used == 1;
+	cout <<"Decisions/Propagations "<<decisions<<"/"<<mems<<endl;
+	cout <<"Conflicts: "<<conflicts<<endl;
+	cout <<"Learned clauses/bin/unit: "<<learned_clauses<<"/"<<learned_bin_clauses<<"/"<<learned_units<<endl;
+	cout <<"Forgot clauses: "<<forgot_clauses<<endl;
+	cout <<"Nontriv redu: "<<nontriv_redu<<endl;
+	cout <<"Restarts "<<restarts<<endl;
 }
 
 void Oracle::AddSolToCache() {
@@ -58,126 +54,56 @@ void Oracle::AddSolToCache() {
 		assert(vs[i].phase == 0 || vs[i].phase == 1);
 		sol_cache[i].push_back(vs[i].phase);
 	}
+    stats.cache_added++;
 }
 
 void Oracle::ClearSolCache() {
-	if (sol_cache[1].size() == 0) return;
-	for (Var v = 1; v <= vars; v++) {
-		sol_cache[v].clear();
-	}
+	if (sol_cache[1].empty()) return;
+	for (Var v = 1; v <= vars; v++) sol_cache[v].clear();
 }
 
-bool Oracle::SatByCache(const vector<Lit>& assumps) {
-// 	stats.cache_timer.start();
+bool Oracle::SatByCache(const vector<Lit>& assumps) const {
+    // 1st variable's cache size is the same as all the rest
 	int cs = sol_cache[1].size();
+
+    // Try all cache lines
 	for (int i = 0; i < cs; i++) {
 		bool ok = true;
-		for (Lit l : assumps) {
+        // all our assumptions must be in the solution
+		for (const Lit& l : assumps) {
 			if (IsPos(l)) {
-				if (sol_cache[VarOf(l)][i] == 0) {
-					ok = false;
-					break;
-				}
+				if (sol_cache[VarOf(l)][i] == 0) { ok = false; break; }
 			} else {
-				if (sol_cache[VarOf(l)][i] == 1) {
-					ok = false;
-					break;
-				}
+				if (sol_cache[VarOf(l)][i] == 1) { ok = false; break; }
 			}
 		}
-		if (ok) {
-// 			stats.cache_timer.stop();
-			return true;
-		}
+		if (ok) return true;
 	}
-// 	stats.cache_timer.stop();
+    // Not in the cache
 	return false;
 }
 
-void Oracle::PrintStats() const {
-	stats.Print();
-}
-
-void Oracle::UpdGlueEma(int glue) {
-	glue_long_ema = ((long double)1.0-long_a)*glue_long_ema + long_a*(long double)glue;
-	glue_short_ema = ((long double)1.0-short_a)*glue_short_ema + short_a*(long double)glue;
-	if (long_a > 1.0/5000.0) {
-		long_a /= 2.0;
-	}
-	if (short_a > 1.0/50.0) {
-		short_a /= 2.0;
-	}
-}
-
-void Oracle::UpdVarAssEma() {
-	var_ass_ema = ((long double)1.0-var_ass_a)*var_ass_ema + var_ass_a*(long double)decided.size();
-	if (var_ass_a > 1.0/5000.0) {
-		var_ass_a /= 2.0;
-	}
-}
-
-void Oracle::ForgetLearned() {
-	assert(CurLevel() == 1);
-	if (cla_info.empty()) return;
-	vector<char> watch_res(vars*2+2);
-	vector<vector<Lit>> to_add;
-	for (const auto& ci : cla_info) {
-		assert(ci.pt >= orig_clauses_size);
-		watch_res[clauses[ci.pt]] = true;
-		watch_res[clauses[ci.pt+1]] = true;
-		if (ci.glue == -1) {
-			assert(ci.used <= -1);
-			// Added clause
-			to_add.push_back({});
-			for (size_t i = ci.pt; clauses[i]; i++) {
-				to_add.back().push_back(clauses[i]);
-			}
-			assert(to_add.back().size() >= 2);
-		}
-	}
-	cla_info.clear();
-	clauses.resize(orig_clauses_size);
-	for (Lit l = 2; l <= vars*2+1; l++) {
-		if (!watch_res[l]) continue;
-		size_t pos = 0;
-		for (size_t i = 0; i < watches[l].size(); i++) {
-			watches[l][pos++] = watches[l][i];
-			if (watches[l][pos-1].cls >= orig_clauses_size) {
-				pos--;
-			}
-		}
-		assert(pos < watches[l].size());
-		watches[l].resize(pos);
-	}
-	for (const auto& clause : to_add) {
-		AddOrigClause(clause, false);
-	}
-	assert(clauses.size() == orig_clauses_size);
-	assert(cla_info.empty());
-}
-
 void Oracle::ResizeClauseDb() {
-	stats.maint_time.start();
 	std::sort(cla_info.begin(), cla_info.end(), [](const CInfo& a, const CInfo& b){
-		if (a.glue == b.glue) {
-			return a.used > b.used;
-		}
-		return a.glue < b.glue;
+        if (a.glue == -1 || b.glue == -1) return a.glue < b.glue;
+		if (a.used != b.used) return a.used > b.used;
+        return a.total_used > b.total_used;
 	});
 	{
 		vector<size_t> new_reason(vars+2);
-		for (Var v = 1; v <= vars; v++) {
-			new_reason[v] = vs[v].reason;
-		}
+		for (Var v = 1; v <= vars; v++) { new_reason[v] = vs[v].reason; }
 		size_t prev_orig_clauses_size = orig_clauses_size;
 		vector<Lit> new_clauses(orig_clauses_size);
-		for (size_t i = 0; i < orig_clauses_size; i++) {
-			new_clauses[i] = clauses[i];
-		}
+		for (size_t i = 0; i < orig_clauses_size; i++) { new_clauses[i] = clauses[i]; }
 		vector<CInfo> new_cla_info;
 		// This sets new_clauses and fixes reasons
-		const size_t good_size = min(cla_info.size()/2, (size_t)1000000);
+		const size_t good_size = 10000;
+        num_lbd2_red_cls = 0;
+        num_used_red_cls = 0;
 		for (size_t i = 0; i < cla_info.size(); i++) {
+            if (i+1 < cla_info.size()) {
+                __builtin_prefetch(clauses.data() + cla_info[i+1].pt);
+            }
 			stats.mems++;
 			Lit impll = 0;
 			size_t cls = cla_info[i].pt;
@@ -201,65 +127,52 @@ void Oracle::ResizeClauseDb() {
 			size_t len = 0;
 			bool frozen_sat = false;
 			while (clauses[cls+len]) {
-				if (LitVal(clauses[cls+len]) == 1) {
+				if (!frozen_sat && LitVal(clauses[cls+len]) == 1) {
 					Var v = VarOf(clauses[cls+len]);
-					if (vs[v].level == 1) {
-						frozen_sat = true;
-					}
+					if (vs[v].level == 1) frozen_sat = true;
 				}
 				len++;
 			}
 			assert(len >= 2);
-			if (frozen_sat) {
-				assert(!impll);
-			}
-			if (frozen_sat || (impll == 0 && !added && !cla_info[i].Keep() && i > good_size)) {
+			if (frozen_sat) assert(!impll);
+            if (cla_info[i].glue <= 2) num_lbd2_red_cls++;
+            else if (cla_info[i].used) num_used_red_cls++;
+			if (frozen_sat || (impll == 0 && !added && !cla_info[i].Keep()
+                        && i > good_size+num_lbd2_red_cls)) {
 				stats.forgot_clauses++;
 				clauses[cls] = 0;
 				continue;
 			}
 			size_t new_pt = new_clauses.size();
-			if (impll) {
-				new_reason[VarOf(impll)] = new_pt;
-			}
-			if (added) {
-				assert(new_clauses.size() == orig_clauses_size);
-			} else {
-				new_cla_info.push_back({new_clauses.size(), cla_info[i].glue, cla_info[i].used-1});
-			}
-			for (size_t k = cls; clauses[k]; k++) {
-				new_clauses.push_back(clauses[k]);
-			}
+			if (impll) new_reason[VarOf(impll)] = new_pt;
+			if (added) assert(new_clauses.size() == orig_clauses_size);
+			else new_cla_info.push_back({new_clauses.size(), cla_info[i].glue, cla_info[i].used-1, cla_info[i].total_used});
+			for (size_t k = cls; clauses[k]; k++) new_clauses.push_back(clauses[k]);
 			new_clauses.push_back(0);
-			if (added) {
-				orig_clauses_size = new_clauses.size();
-			}
+			if (added) orig_clauses_size = new_clauses.size();
 			clauses[cls] = new_pt;
 		}
-		for (Var v = 1; v <= vars; v++) {
-			vs[v].reason = new_reason[v];
-		}
+		for (Var v = 1; v <= vars; v++) vs[v].reason = new_reason[v];
 		for (Lit l = 2; l <= vars*2+1; l++) {
 			size_t pos = 0;
 			for (size_t i = 0; i < watches[l].size(); i++) {
 				watches[l][pos++] = watches[l][i];
 				if (watches[l][pos-1].cls >= prev_orig_clauses_size) {
-					if (clauses[watches[l][pos-1].cls] == 0) {
-						pos--;
-					} else {
-						watches[l][pos-1].cls = clauses[watches[l][pos-1].cls];
-					}
+					if (clauses[watches[l][pos-1].cls] == 0) pos--;
+					else watches[l][pos-1].cls = clauses[watches[l][pos-1].cls];
 				}
 			}
 			watches[l].resize(pos);
 		}
 		clauses = new_clauses;
 		cla_info = new_cla_info;
+#ifdef SLOW_DEBUG
 		for (Lit l = 2; l <= vars*2+1; l++) {
-			for (Watch w : watches[l]) {
+			for (const auto& Watch w : watches[l]) {
 				assert(clauses[w.cls] == l || clauses[w.cls+1] == l);
 			}
 		}
+#endif
 	}
 	for (Var v = 1; v <= vars; v++) {
 		if (vs[v].reason) {
@@ -273,14 +186,10 @@ void Oracle::ResizeClauseDb() {
 			}
 		}
 	}
-	stats.maint_time.stop();
 }
 
 void Oracle::BumpClause(size_t cls) {
-	if (cls < orig_clauses_size) {
-		og_bump++;
-		return;
-	}
+	if (cls < orig_clauses_size) return;
 	assert(cla_info.size() > 0);
 	size_t i = 0;
 	for (size_t b = cla_info.size()/2; b >= 1; b /= 2) {
@@ -290,7 +199,7 @@ void Oracle::BumpClause(size_t cls) {
 	}
 	assert(cla_info[i].pt == cls);
 	if (cla_info[i].glue == -1) {
-		// Special added clause 
+		// Special added clause
 		assert(cla_info[i].used == -1);
 		return;
 	}
@@ -304,6 +213,7 @@ void Oracle::BumpClause(size_t cls) {
 	}
 	cla_info[i].glue = glue;
 	cla_info[i].used = 1;
+	cla_info[i].total_used++;
 	return;
 }
 
@@ -384,25 +294,6 @@ void Oracle::BumpVar(Var v) {
 	}
 }
 
-int Oracle::LitVal(Lit lit) const {
-	return lit_val[lit];
-}
-
-bool Oracle::LitSat(Lit lit) const {
-	return LitVal(lit) > 0;
-}
-
-bool Oracle::LitAssigned(Lit lit) const {
-	return LitVal(lit) != 0;
-}
-
-int Oracle::CurLevel() const {
-	if (decided.empty()) {
-		return 1;
-	}
-	return vs[decided.back()].level;
-}
-
 void Oracle::SetAssumpLit(Lit lit, bool freeze) {
 	assert(CurLevel() == 1);
 	Var v = VarOf(lit);
@@ -434,11 +325,8 @@ void Oracle::SetAssumpLit(Lit lit, bool freeze) {
 	assert(watches[lit].empty());
 	assert(watches[Neg(lit)].empty());
 	assert(prop_q.empty());
-	if (freeze) {
-		Assign(lit, 0, 1);
-	} else {
-		Assign(lit, 0, 2);
-	}
+	if (freeze) { Assign(lit, 0, 1); }
+	else { Assign(lit, 0, 2); }
 	assert(decided.back() == VarOf(lit));
 	decided.pop_back();
 	assert(prop_q.back() == Neg(lit));
@@ -446,27 +334,17 @@ void Oracle::SetAssumpLit(Lit lit, bool freeze) {
 }
 
 void Oracle::Assign(Lit dec, size_t reason_clause, int level) {
-	if (level <= 1) {
-		reason_clause = 0;
-	}
+	if (level <= 1) reason_clause = 0;
 	Var v = VarOf(dec);
 	lit_val[dec] = 1;
 	lit_val[Neg(dec)] = -1;
-	if (IsPos(dec)) {
-		vs[v].phase = 1;
-	} else {
-		vs[v].phase = 0;
-	}
+    vs[v].phase = IsPos(dec) ? 1 : 0;
 	vs[v].reason = reason_clause;
 	vs[v].level = level;
+    oclv("Assigning " << v << " to: " << IsPos(dec) << " at level: " << level << " reason: " << reason_clause);
 	decided.push_back(v);
 	prop_q.push_back(Neg(dec));
-}
-
-void Oracle::Decide(Lit dec, int level) {
-	assert(LitVal(dec) == 0);
-	stats.decisions++;
-	Assign(dec, 0, level);
+    __builtin_prefetch(watches[Neg(dec)].data());
 }
 
 void Oracle::UnDecide(int level) {
@@ -479,16 +357,18 @@ void Oracle::UnDecide(int level) {
 		vs[v].reason = 0;
 		vs[v].level = 0;
 		ActivateActivity(v);
+        oclv("UNAss " << v);
 	}
 	assert(prop_q.empty());
 }
 
 size_t Oracle::AddLearnedClause(const vector<Lit>& clause) {
 	stats.learned_clauses++;
-	if (clause.size() == 2) {
-		stats.learned_bin_clauses++;
-	}
+	if (clause.size() == 2) stats.learned_bin_clauses++;
 	assert(clause.size() >= 2);
+
+    // Compute LBD. Notice that the clause has been sorted already in terms of decision
+    // levels of its literals, before this function is called.
 	int glue = 2;
 	assert(!LitAssigned(clause[0]));
 	for (size_t i = 1; i < clause.size(); i++) {
@@ -500,15 +380,14 @@ size_t Oracle::AddLearnedClause(const vector<Lit>& clause) {
 			}
 		}
 	}
+
+    // Attach
 	size_t pt = clauses.size();
 	watches[clause[0]].push_back({pt, clause[1], (int)clause.size()});
 	watches[clause[1]].push_back({pt, clause[0], (int)clause.size()});
-	for (Lit lit : clause) {
-		clauses.push_back(lit);
-	}
+	for (Lit lit : clause) clauses.push_back(lit);
 	clauses.push_back(0);
-	cla_info.push_back({pt, glue, 0});
-	UpdGlueEma(glue);
+	cla_info.push_back({pt, glue, 1, 0});
 	return pt;
 }
 
@@ -553,14 +432,25 @@ bool Oracle::LitReduntant(Lit lit) {
 }
 
 vector<Lit> Oracle::LearnUip(size_t conflict_clause) {
-// 	stats.learn_timer.start();
 	assert(conflict_clause > 0);
 	BumpClause(conflict_clause);
+#ifdef DEBUG_ORACLE_VERB
+    oclv2("Conflict clause NUM: " << conflict_clause << " cl:");
+    for(size_t i = conflict_clause; clauses[i]; i++) {
+        oclv2(" v: " << VarOf(clauses[i]) << " val: " << LitVal(clauses[i]));
+    }
+    oclv2(endl);
+#endif
+
 	vector<Lit> clause = {0};
 	int level = vs[VarOf(clauses[conflict_clause])].level;
 	int open = 0;
+    oclv("---");
 	for (size_t i = conflict_clause; clauses[i]; i++) {
 		assert(LitVal(clauses[i]) == -1);
+        oclv("clauses[i]: " << VarOf(clauses[i]) << " val: " << LitVal(clauses[i])
+                << " vs[VarOf(clauses[i])].level: " << vs[VarOf(clauses[i])].level
+                << " level: " << level);
 		assert(vs[VarOf(clauses[i])].level <= level);
 		BumpVar(VarOf(clauses[i]));
 		if (vs[VarOf(clauses[i])].level == level) {
@@ -598,8 +488,11 @@ vector<Lit> Oracle::LearnUip(size_t conflict_clause) {
 		seen[v] = false;
 	}
 	for (size_t i = 1; i < clause.size(); i++) {
-		assert(VarOf(clause[i]) != VarOf(clause[i-1]));	
+		assert(VarOf(clause[i]) != VarOf(clause[i-1]));
 	}
+
+
+    // Conflict minimization
 	for (size_t i = 1; i < clause.size(); i++) {
 		if (vs[VarOf(clause[i])].reason) {
 			stats.mems++;
@@ -623,13 +516,11 @@ vector<Lit> Oracle::LearnUip(size_t conflict_clause) {
 		assert(in_cc[clause[i]]);
 		in_cc[clause[i]] = false;
 	}
-// 	stats.learn_timer.stop();
 	return clause;
 }
 
 // Returns id of conflict clause or 0 if no conflict
 size_t Oracle::Propagate(int level) {
-// 	stats.prop_timer.start();
 	size_t conflict = 0;
 	for (size_t i = 0; i < prop_q.size(); i++) {
 		stats.mems++;
@@ -702,7 +593,6 @@ size_t Oracle::Propagate(int level) {
 	}
 	//stats.propagations += (int)prop_q.size();
 	prop_q.clear();
-// 	stats.prop_timer.stop();
 	return conflict;
 }
 
@@ -722,12 +612,15 @@ int Oracle::CDCLBT(size_t confl_clause, int min_level) {
 		int ass_level = vs[VarOf(clause[1])].level;
 		assert(ass_level >= 2);
 		assert(ass_level < vs[VarOf(clause[0])].level);
+        oclv("ass_level: " << ass_level << " min_level: " << min_level);
 		if (ass_level >= min_level) {
+            oclv("if (ass_level >= min_level) ");
 			UnDecide(ass_level+1);
 			size_t cl_id = AddLearnedClause(clause);
 			Assign(clause[0], cl_id, ass_level);
 			return ass_level;
 		} else {
+            oclv("NOT if (ass_level >= min_level)");
 			assert(prop_q.empty());
 			UnDecide(min_level+1);
 			vector<pair<Lit, int>> decs;
@@ -743,24 +636,22 @@ int Oracle::CDCLBT(size_t confl_clause, int min_level) {
 			UnDecide(ass_level+1);
 			size_t cl_id = AddLearnedClause(clause);
 			Assign(clause[0], cl_id, ass_level);
-			if (Propagate(ass_level)) {
-				return min_level-1;
-			}
+			if (Propagate(ass_level)) return min_level-1;
 			int level = ass_level;
 			std::reverse(decs.begin(), decs.end());
 			for (int i = 0; i < (int)decs.size(); i++) {
-				if (LitVal(decs[i].F) == -1) {
+				if (LitVal(decs[i].first) == -1) {
 					return min_level-1;
 				}
-				if (LitVal(decs[i].F) == 0) {
-					Decide(decs[i].F, decs[i].S);
-					if (Propagate(decs[i].S)) {
+				if (LitVal(decs[i].first) == 0) {
+					Decide(decs[i].first, decs[i].second);
+					if (Propagate(decs[i].second)) {
 						return min_level-1;
 					}
-					level = decs[i].S;
+					level = decs[i].second;
 				}
 				if (i) {
-					assert(decs[i].S >= decs[i-1].S);
+					assert(decs[i].second >= decs[i-1].second);
 				}
 			}
 			return max(level, min_level);
@@ -768,72 +659,54 @@ int Oracle::CDCLBT(size_t confl_clause, int min_level) {
 	}
 }
 
-
 TriState Oracle::HardSolve(int64_t max_mems) {
 	InitLuby();
 	int64_t confls = 0;
 	int64_t next_restart = 1;
-	int64_t next_db_clean = 1;
-	int64_t db_cleans = 0;
-	glue_long_ema = 0;
-	glue_short_ema = 0;
-	var_ass_ema = 0;
-	var_ass_a = 1;
-	long_a = 1;
-	short_a = 1;
 	int64_t mems_startup = stats.mems;
 	int cur_level = 2;
-	int last_var_ass_size = 0;
 	Var nv = 1;
 	while (true) {
 		size_t confl_clause = Propagate(cur_level);
 		if (stats.mems > mems_startup+max_mems) return TriState::unknown();
 		if (confl_clause) {
 			confls++;
-			UpdVarAssEma();
-			last_var_ass_size = decided.size();
-			if (cur_level <= 2) {
-				return false;
-			}
+            total_confls++;
+			if (cur_level <= 2) return false;
 			cur_level = CDCLBT(confl_clause);
 			assert(cur_level >= 2);
 			continue;
 		}
-		//if (stats.conflicts > next_restart) {
 		if (confls >= next_restart) {
-			// cerr<<"restart "<<cla_info.size()<<" "<<glue_long_ema<<" "<<glue_short_ema<<" "<<var_ass_ema<<" "<<last_var_ass_size<<" "<<og_bump<<endl;
 			int nl = NextLuby();
 			next_restart = confls + nl*restart_factor;
 			UnDecide(3);
 			cur_level = 2;
 			stats.restarts++;
-			if (confls >= next_db_clean) {
-				db_cleans++;
-				//next_db_clean = stats.conflicts + 500*(db_cleans*db_cleans+3);
-				next_db_clean = confls + ideal_clause_db_size;
-				//cerr<<"resize clause DB "<<cla_info.size()<<endl;
+			if (total_confls > last_db_clean + 10000) {
+                last_db_clean = total_confls;
+                oclv("c [oracle] Resizing cldb"
+                    << " num_lbd2_red_cls: " << num_lbd2_red_cls
+                    << " num_used_red_cls: " << num_used_red_cls
+                    << " cla_info.size(): " << cla_info.size());
 				ResizeClauseDb();
+                oclv("c [oracle] after cldb resize"
+                    << " num_lbd2_red_cls: " << num_lbd2_red_cls
+                    << " num_used_red_cls: " << num_used_red_cls
+                    << " cla_info.size(): " << cla_info.size());
 			}
 		}
 		Var decv = 0;
 		if (confls == 0) {
-			while (nv <= vars && LitVal(PosLit(nv)) != 0) {
-				nv++;
-			}
-			if (nv <= vars) {
-				decv = nv;
-			}
+			while (nv <= vars && LitVal(PosLit(nv)) != 0) nv++;
+			if (nv <= vars) decv = nv;
 		} else {
-			while (1) {
+			while (true) {
 				decv = PopVarHeap();
-				if (decv == 0 || LitVal(PosLit(decv)) == 0) {
-					break;
-				}
+				if (decv == 0 || LitVal(PosLit(decv)) == 0) break;
 			}
 		}
-		if (decv == 0) {
-			return true;
-		}
+		if (decv == 0) return true;
 		cur_level++;
 		Decide(MkLit(decv, vs[decv].phase), cur_level);
 	}
@@ -849,12 +722,8 @@ void Oracle::AddOrigClause(vector<Lit> clause, bool entailed) {
 			i--;
 		}
 	}
-	for (Lit lit : clause) {
-		assert(LitVal(lit) == 0);
-	}
-	if (!entailed) {
-		ClearSolCache();
-	}
+	for (Lit lit : clause) assert(LitVal(lit) == 0);
+	if (!entailed) ClearSolCache();
 	if (clause.size() == 0) {
 		unsat = true;
 		return;
@@ -868,9 +737,7 @@ void Oracle::AddOrigClause(vector<Lit> clause, bool entailed) {
 	size_t pt = clauses.size();
 	watches[clause[0]].push_back({clauses.size(), clause[1], (int)clause.size()});
 	watches[clause[1]].push_back({clauses.size(), clause[0], (int)clause.size()});
-	for (Lit lit : clause) {
-		clauses.push_back(lit);
-	}
+	for (Lit lit : clause) clauses.push_back(lit);
 	clauses.push_back(0);
 	// If we have no learned clauses then this is original clause
 	// Otherwise this is "learned" clause with glue -1 and used -1
@@ -878,18 +745,18 @@ void Oracle::AddOrigClause(vector<Lit> clause, bool entailed) {
 		orig_clauses_size = clauses.size();
 		return;
 	}
-	cla_info.push_back({pt, -1, -1});
+	cla_info.push_back({pt, -1, -1, 0});
 }
 
-Oracle::Oracle(int vars_, const vector<vector<Lit>>& clauses_, const vector<vector<Lit>>& learned_clauses_) : Oracle(vars_, clauses_) {
+Oracle::Oracle(int vars_, const vector<vector<Lit>>& clauses_,
+        const vector<vector<Lit>>& learned_clauses_) : Oracle(vars_, clauses_) {
 	for (const auto& clause : learned_clauses_) {
-		AddClauseIfNeeded(clause, true);
+		AddClauseIfNeededAndStr(clause, true);
 	}
 }
 
 Oracle::Oracle(int vars_, const vector<vector<Lit>>& clauses_) : vars(vars_), rand_gen(1337) {
 	assert(vars >= 1);
-	stats.setup_timer.start();
 	vs.resize(vars+1);
 	seen.resize(vars+1);
 	lvl_seen.resize(vars+3);
@@ -899,7 +766,6 @@ Oracle::Oracle(int vars_, const vector<vector<Lit>>& clauses_) : vars(vars_), ra
 	redu_seen.resize(vars*2+2);
 	in_cc.resize(vars*2+2);
 	// setting magic constants
-	ideal_clause_db_size = vars*4;
 	restart_factor = 100;
 
 	clauses.push_back(0);
@@ -909,12 +775,10 @@ Oracle::Oracle(int vars_, const vector<vector<Lit>>& clauses_) : vars(vars_), ra
 	}
 
 	// variable activity
-	var_fact = powl((long double)2, (long double)(1.0/(long double)vars));
+	var_fact = powl((long double)2, (long double)(1.0L/(long double)vars));
 	assert(var_fact > 1.0);
 	heap_N = 1;
-	while (heap_N <= (size_t)vars) {
-		heap_N *= 2;
-	}
+	while (heap_N <= (size_t)vars) heap_N *= 2;
 	var_act_heap.resize(heap_N*2);
 	for (Var v = 1; v <= vars; v++) {
 		var_act_heap[heap_N + v] = var_inc * RandInt(95, 105, rand_gen);
@@ -922,50 +786,25 @@ Oracle::Oracle(int vars_, const vector<vector<Lit>>& clauses_) : vars(vars_), ra
 	for (int i = heap_N-1; i >= 1; i--) {
 		var_act_heap[i] = max(var_act_heap[i*2], var_act_heap[i*2+1]);
 	}
-	stats.setup_timer.stop();
-}
-
-bool Oracle::FalseByProp(const vector<Lit>& assumps) {
-	if (unsat) return true;
-	for (Lit lit : assumps) {
-		if (LitVal(lit) == -1) {
-			return true;
-		}
-	}
-	for (Lit lit : assumps) {
-		if (LitVal(lit) == 0) {
-			Decide(lit, 2);
-		}
-	}
-	size_t confl_clause = Propagate(2);
-	UnDecide(2);
-	if (confl_clause && assumps.size() == 1) {
-		FreezeUnit(Neg(assumps[0]));
-	}
-	return confl_clause;
 }
 
 TriState Oracle::Solve(const vector<Lit>& assumps, bool usecache, int64_t max_mems) {
 	if (unsat) return false;
-	if (usecache && SatByCache(assumps)) return true;
-	stats.solve_timer.start();
+	if (usecache && SatByCache(assumps)) {stats.cache_useful++; return true;}
+    oclv("SOLVE called ");
 	// TODO: solution caching
-	for (Lit lit : assumps) {
+	for (const auto& lit : assumps) {
 		if (LitVal(lit) == -1) {
 			prop_q.clear();
 			UnDecide(2);
-			stats.solve_timer.stop();
 			return false;
 		} else if (LitVal(lit) == 0) {
 			Decide(lit, 2);
 		}
 	}
 	size_t confl_clause = Propagate(2);
-	if (confl_clause) {
-		UnDecide(2);
-		stats.solve_timer.stop();
-		return false;
-	}
+	if (confl_clause) { UnDecide(2); return false; }
+    oclv("HARD SOLVING");
 	TriState sol = HardSolve(max_mems);
 	UnDecide(2);
 	if (!unsat) {
@@ -979,9 +818,7 @@ TriState Oracle::Solve(const vector<Lit>& assumps, bool usecache, int64_t max_me
 		}
 	}
 	if (sol.isTrue()) {
-		if (usecache) {
-			AddSolToCache();
-		}
+		if (usecache) { AddSolToCache(); }
 	} else if (sol.isFalse()) {
 		// UNSAT
 		if (assumps.size() == 1) {
@@ -989,124 +826,11 @@ TriState Oracle::Solve(const vector<Lit>& assumps, bool usecache, int64_t max_me
 			if (!ok) {
 				assert(unsat);
 			}
-		} else {
-			//assert(FalseByProp(assumps));
 		}
 	}
-	stats.solve_timer.stop();
 	return sol;
 }
 
-int Oracle::PropDg(const vector<Lit>& assumps) {
-	if (unsat) return vars;
-	for (Lit lit : assumps) {
-		if (LitVal(lit) == -1) {
-			prop_q.clear();
-			UnDecide(2);
-			return vars;
-		}
-		if (LitVal(lit) == 0) {
-			Decide(lit, 2);
-		}
-	}
-	size_t confl_clause = Propagate(2);
-	if (confl_clause) {
-		UnDecide(2);
-		return vars;
-	}
-	int ret = 0;
-	for (Var v = 1; v <= vars; v++) {
-		if (LitVal(PosLit(v)) != 0) {
-			ret++;
-		}
-	}
-	UnDecide(2);
-	return ret;
-}
-
-vector<Lit> Oracle::InferUnits(const vector<Lit>& assumps) {
-	if (unsat) return {NegLit(1), PosLit(1)};
-	for (Lit lit : assumps) {
-		if (LitVal(lit) == -1) {
-			prop_q.clear();
-			UnDecide(2);
-			return {NegLit(1), PosLit(1)};
-		}
-		if (LitVal(lit) == 0) {
-			Decide(lit, 2);
-		}
-	}
-	size_t confl_clause = Propagate(2);
-	if (confl_clause) {
-		UnDecide(2);
-		return {NegLit(1), PosLit(1)};
-	}
-	auto sol = HardSolve();
-	assert(!sol.isUnknown());
-	if (sol.isFalse()) {
-		UnDecide(2);
-		return {NegLit(1), PosLit(1)};
-	}
-	vector<char> sol_val(vars+1);
-	for (Var v = 1; v <= vars; v++) {
-		sol_val[v] = vs[v].phase;
-	}
-	UnDecide(3);
-	for (Lit lit : assumps) {
-		assert(LitVal(lit) == 1);
-	}
-	// Failed literal propagation
-	bool fo = true;
-	while (fo) {
-		fo = false;
-		for (Var v = 1; v <= vars; v++) {
-			Lit lit = MkLit(v, sol_val[v]);
-			if (LitVal(lit) != 0) {
-				assert(LitVal(lit) == 1);
-				continue;
-			}
-			Decide(Neg(lit), 3);
-			confl_clause = Propagate(3);
-			UnDecide(3);
-			if (confl_clause) {
-				Decide(lit, 2);
-				assert(Propagate(2) == 0);
-			}
-		}
-	}
-	// Collect
-	vector<Lit> ret;
-	for (Var v = 1; v <= vars; v++) {
-		Lit lit = MkLit(v, sol_val[v]);
-		if (LitVal(lit) != 0) {
-			assert(LitVal(lit) == 1);
-			ret.push_back(lit);
-			continue;
-		}
-	}
-	UnDecide(2);
-	return ret;
-}
-
-bool Oracle::FreezeUnits(const vector<Lit>& units) {
-	if (unsat) return false;
-	assert(CurLevel() == 1);
-	for (Lit unit : units) {
-		if (LitVal(unit) == -1) {
-			return false;
-		}
-		if (LitVal(unit) == 0) {
-			Decide(unit, 1);
-			stats.learned_units++;
-		}
-	}
-	size_t confl = Propagate(1);
-	if (confl) {
-		unsat = true;
-	}
-	return confl == 0;
-}
-
 bool Oracle::FreezeUnit(Lit unit) {
 	if (unsat) return false;
 	assert(CurLevel() == 1);
@@ -1119,6 +843,7 @@ bool Oracle::FreezeUnit(Lit unit) {
 	}
 	Decide(unit, 1);
 	stats.learned_units++;
+    oclv("[oracle] learnt unit: " << VarOf(unit) << " val: " << LitVal(unit));
 	size_t confl = Propagate(1);
 	if (confl) {
 		unsat = true;
@@ -1127,11 +852,9 @@ bool Oracle::FreezeUnit(Lit unit) {
 	return true;
 }
 
-void Oracle::AddClause(const vector<Lit>& clause, bool entailed) {
-	AddOrigClause(clause, entailed);
-}
-
-bool Oracle::AddClauseIfNeeded(vector<Lit> clause, bool entailed) {
+// Checks if the clause is SAT & then does strengthening on the clause
+// before adding it to the CNF
+bool Oracle::AddClauseIfNeededAndStr(vector<Lit> clause, bool entailed) {
 	if (unsat) return false;
 	assert(CurLevel() == 1);
 	for (int i = 0; i < (int)clause.size(); i++) {
@@ -1146,6 +869,10 @@ bool Oracle::AddClauseIfNeeded(vector<Lit> clause, bool entailed) {
 		AddOrigClause(clause, entailed);
 		return true;
 	}
+
+    // We are now going to do strengthening, lit-by-lit, recursively.
+    // For example, a V b V c. Enqueue (!a && !b) and propagate. If UNSAT,
+    // then we know (a V b) is also true. We just removed a literal.
 	for (int i = 0; i < (int)clause.size(); i++) {
 		Lit tp = clause[i];
 		assert(LitVal(tp) == 0);
@@ -1158,13 +885,15 @@ bool Oracle::AddClauseIfNeeded(vector<Lit> clause, bool entailed) {
 		if (confl || LitVal(tp) == -1) {
 			UnDecide(2);
 			SwapDel(clause, i);
-			return AddClauseIfNeeded(clause, true);
+			return AddClauseIfNeededAndStr(clause, true);
 		}
+
+        // No conflict.
 		if (LitVal(tp) == 1) {
 			// Propagation as intended
 			UnDecide(2);
 		} else if (LitVal(tp) == 0) {
-			// No propagation
+			// No propagation -- we need to add the clause
 			UnDecide(2);
 			AddOrigClause(clause, entailed);
 			return true;
@@ -1175,38 +904,7 @@ bool Oracle::AddClauseIfNeeded(vector<Lit> clause, bool entailed) {
 	return false;
 }
 
-double Oracle::ConflictRate(int samples) {
-	if (unsat) {
-		return 0;
-	}
-	vector<Var> var_list;
-	for (Var v = 1; v <= vars; v++) {
-		if (LitVal(PosLit(v)) == 0) {
-			var_list.push_back(v);
-		}
-	}
-	int confls = 0;
-	for (int sample = 0; sample < samples; sample++) {
-		Shuffle(var_list, rand_gen);
-		bool cf = false;
-		for (Var v : var_list) {
-			if (LitVal(PosLit(v)) != 0) continue;
-			Decide(MkLit(v, RandBool(rand_gen)), 2);
-			size_t confl = Propagate(2);
-			if (confl) {
-				cf = true;
-				break;
-			}
-		}
-		UnDecide(2);
-		if (cf) {
-			confls++;
-		}
-	}
-	return (double)confls/(double)samples;
-}
-
-vector<vector<Lit>> Oracle::LearnedClauses() const {
+vector<vector<Lit>> Oracle::GetLearnedClauses() const {
 	assert(CurLevel() == 1);
 	vector<vector<Lit>> ret;
 	ret.push_back({});
@@ -1221,6 +919,8 @@ vector<vector<Lit>> Oracle::LearnedClauses() const {
 	}
 	assert(ret.back().empty());
 	ret.pop_back();
+
+    // Units
 	for (Var v = 1; v <= vars; v++) {
 		if (LitVal(PosLit(v)) == 1) {
 			ret.push_back({PosLit(v)});
@@ -1231,22 +931,5 @@ vector<vector<Lit>> Oracle::LearnedClauses() const {
 	return ret;
 }
 
-vector<vector<Lit>> Oracle::AllClauses() const {
-	vector<vector<Lit>> ret = LearnedClauses();
-	ret.push_back({});
-	for (size_t i = 1; i < orig_clauses_size; i++) {
-		if (clauses[i] == 0) {
-			assert(ret.back().size() >= 2);
-			sort(ret.back().begin(), ret.back().end());
-			ret.push_back({});
-		} else {
-			ret.back().push_back(clauses[i]);
-		}
-	}
-	assert(ret.back().empty());
-	ret.pop_back();
-	return ret;
-}
-
 } // namespace oracle
 } // namespace sspp
