@@ -20,8 +20,7 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
 ***********************************************/
 
-#ifndef __PROPENGINE_H__
-#define __PROPENGINE_H__
+#pragma once
 
 // #define VERBOSE_DEBUG
 #include <cstdio>
@@ -46,7 +45,6 @@ THE SOFTWARE.
 
 namespace CMSat {
 
-using std::set;
 class Solver;
 class SQLStats;
 class DataSync;
@@ -742,5 +740,3 @@ inline void PropEngine::attach_bin_clause(
 }
 
 } //end namespace
-
-#endif //__PROPENGINE_H__
