@@ -20,13 +20,13 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
 ***********************************************/
 
-#ifndef SOLVER_H
-#define SOLVER_H
+#pragma once
 
 #include "constants.h"
 #include <vector>
 #include <fstream>
 #include <iostream>
+#include <array>
 #include <utility>
 #include <string>
 #include <algorithm>
@@ -42,12 +42,18 @@ THE SOFTWARE.
 #ifdef STATS_NEEDED
 #include "satzilla_features.h"
 #endif
+#define ORACLE_DAT_SIZE 4
 
 namespace CMSat {
-
+    struct SATSolver;
+}
 using std::vector;
 using std::pair;
 using std::string;
+using std::array;
+struct PicoSAT;
+
+namespace CMSat {
 
 class VarReplacer;
 class ClauseCleaner;
@@ -88,7 +94,7 @@ class Solver : public Searcher
         const vector<std::pair<string, string> >& get_sql_tags() const;
         void new_external_var();
         void new_external_vars(size_t n);
-        bool add_clause_outside(const vector<Lit>& lits);
+        bool add_clause_outside(const vector<Lit>& lits, bool red = false);
         bool add_xor_clause_outside(const vector<uint32_t>& vars, bool rhs);
         bool add_bnn_clause_outside(
             const vector<Lit>& lits,
@@ -125,7 +131,7 @@ class Solver : public Searcher
         void start_getting_small_clauses(
             uint32_t max_len, uint32_t max_glue, bool red = true,
             bool bva_vars = false, bool simplified = false);
-        bool get_next_small_clause(std::vector<Lit>& out, bool all_in_one);
+        bool get_next_small_clause(std::vector<Lit>& out, bool all_in_one = false);
         void end_getting_small_clauses();
         void get_all_irred_clauses(vector<Lit>& out);
         vector<uint32_t> translate_sampl_set(const vector<uint32_t>& sampl_set);
@@ -150,8 +156,6 @@ class Solver : public Searcher
         const SolverConf& getConf() const;
         void setConf(const SolverConf& conf);
         const BinTriStats& getBinTriStats() const;
-        size_t   get_num_long_irred_cls() const;
-        size_t   get_num_long_red_cls() const;
         size_t get_num_vars_elimed() const;
         uint32_t num_active_vars() const;
         void print_mem_stats() const;
@@ -162,7 +166,7 @@ class Solver : public Searcher
         void check_implicit_stats(const bool onlypairs = false) const;
         void check_stats(const bool allowFreed = false) const;
         void reset_vsids();
-
+        bool minimize_clause(vector<Lit>& cl);
 
         //Checks
         void check_implicit_propagated() const;
@@ -195,11 +199,11 @@ class Solver : public Searcher
         void update_assumptions_after_varreplace();
 
         //State load/unload
-        template<typename A>
-        void parse_v_line(A* in, const size_t lineNum);
-        lbool load_solution_from_file(const string& fname);
+        string serialize_solution_reconstruction_data() const;
+        void create_from_solution_reconstruction_data(const string& str);
+        pair<lbool, vector<lbool>> extend_minimized_model(const vector<lbool>& m);
 
-        uint64_t getNumLongClauses() const;
+        // Clauses
         bool add_clause_outer_copylits(const vector<Lit>& ps);
         bool add_xor_clause_inter(
             const vector< Lit >& lits
@@ -291,8 +295,8 @@ class Solver : public Searcher
         vector<pair<Lit, Lit> > get_all_binary_xors() const;
         vector<Xor> get_recovered_xors(const bool xor_together_xors);
         vector<uint32_t> remove_definable_by_irreg_gate(const vector<uint32_t>& vars);
-        void find_equiv_subformula(
-            vector<uint32_t>& sampl_vars, vector<uint32_t>& empty_vars, const bool mirror_empty);
+        void clean_sampl_and_get_empties(
+            vector<uint32_t>& sampl_vars, vector<uint32_t>& empty_vars);
 
         bool remove_and_clean_all();
         vector<Lit> get_toplevel_units_internal(bool outer_numbering) const;
@@ -331,6 +335,10 @@ class Solver : public Searcher
         void renumber_xors_to_outside(const vector<Xor>& xors, vector<Xor>& xors_ret);
         void testing_set_solver_not_fresh();
         bool full_probe(const bool bin_only);
+        PicoSAT* build_picosat();
+        void copy_to_simp(SATSolver* s2);
+        bool backbone_simpl(int64_t max_confl, bool cmsgen, bool& finished);
+        bool removed_var_ext(uint32_t var) const;
 
     private:
         friend class ClauseDumper;
@@ -341,7 +349,6 @@ class Solver : public Searcher
         //FRAT
         void write_final_frat_clauses();
 
-
         struct OracleBin {
             OracleBin (const Lit _l1, const Lit _l2, const int32_t _ID):
                 l1(_l1), l2(_l2), ID(_ID) {}
@@ -354,31 +361,36 @@ class Solver : public Searcher
         };
 
         struct OracleDat {
-            OracleDat(vector<int>& _val, ClOffset _off) :
-                val(_val), off(_off) {which = 0;}
-            OracleDat(vector<int>& _val, OracleBin _bin) :
-                val(_val), bin(_bin) {which = 1;}
+            OracleDat(array<int, ORACLE_DAT_SIZE>& _val, ClOffset _off) :
+                val(_val), off(_off) {binary = 0;}
+            OracleDat(array<int, ORACLE_DAT_SIZE>& _val, OracleBin _bin) :
+                val(_val), bin(_bin) {binary = 1;}
 
-
-            vector<int> val;
+            array<int, ORACLE_DAT_SIZE> val;
             ClOffset off;
             OracleBin bin;
-            int which;
+            int binary;
 
             bool operator<(const OracleDat& other) const {
                 return val < other.val;
             }
         };
+
+        vector<vector<int>> get_irred_cls_for_oracle() const;
+        vector<vector<uint16_t>> compute_edge_weights() const;
+        vector<OracleDat> order_clauses_for_oracle() const;
         void dump_cls_oracle(const string fname, const vector<OracleDat>& cs);
         bool find_equivs();
         bool oracle_vivif(bool& finished);
-        bool sparsify();
+        bool oracle_sparsify();
+        void print_cs_ordering(const vector<OracleDat>& cs) const;
         template<bool bin_only> lbool probe_inter(const Lit l, uint32_t& min_props);
         void reset_for_solving();
         vector<Lit> add_clause_int_tmp_cl;
         lbool iterate_until_solved();
         uint64_t mem_used_vardata() const;
         uint64_t calc_num_confl_to_do_this_iter(const size_t iteration_num) const;
+        void detach_and_free_all_irred_cls();
 
         bool sort_and_clean_clause(
             vector<Lit>& ps
@@ -494,7 +506,7 @@ class Solver : public Searcher
         /////////////////////
         // Clauses
         bool addClauseHelper(vector<Lit>& ps);
-        bool add_clause_outer(vector<Lit>& ps);
+        bool add_clause_outer(vector<Lit>& ps, bool red = false);
 
         /////////////////
         // Debug
@@ -509,11 +521,6 @@ inline void Solver::set_decision_var(const uint32_t var)
     insert_var_order_all(var);
 }
 
-inline uint64_t Solver::getNumLongClauses() const
-{
-    return longIrredCls.size() + longRedCls.size();
-}
-
 inline const SearchStats& Solver::get_stats() const
 {
     return sumSearchStats;
@@ -524,16 +531,6 @@ inline const SolveStats& Solver::get_solve_stats() const
     return solveStats;
 }
 
-inline size_t Solver::get_num_long_irred_cls() const
-{
-    return longIrredCls.size();
-}
-
-inline size_t Solver::get_num_long_red_cls() const
-{
-    return longRedCls.size();
-}
-
 inline const SolverConf& Solver::getConf() const
 {
     return conf;
@@ -695,6 +692,11 @@ inline void Solver::free_cl(
     cl_alloc.clauseFree(offs);
 }
 
-} //end namespace
+inline bool Solver::removed_var_ext(uint32_t var) const
+{
+    assert(get_num_bva_vars() == 0);
+    var = map_outer_to_inter(var);
+    return value(var) != l_Undef || varData[var].removed != Removed::none;
+}
 
-#endif //SOLVER_H
+} //end namespace
