@@ -26,6 +26,7 @@ THE SOFTWARE.
 #include <iomanip>
 #include <sstream>
 using namespace CMSat;
+using std::numeric_limits;
 
 //UPDATEs for DEVEL
 //Fixing to:
@@ -297,28 +298,6 @@ DLL_PUBLIC SolverConf::SolverConf() :
             "louvain-comms,"
         )
 
-        , simplify_schedule_external(
-            "scc-vrepl,"
-            "sub-impl,"
-            "intree-probe,"
-            "sub-str-cls-with-bin,distill-cls,distill-bins,"
-            "scc-vrepl,sub-impl,str-impl,sub-impl,"
-            "breakid,"
-            //occurrence based
-            "occ-backw-sub-str,occ-clean-implicit,occ-bve,"//occ-gates,"
-            "occ-bva,occ-ternary-res,occ-xor,card-find,"
-            //consolidate after OCC
-            "cl-consolidate,"
-            //strengthen again
-            "str-impl,sub-str-cls-with-bin,distill-cls,distill-bins,"
-            "scc-vrepl,"
-            //renumber then it's time for SLS
-            "renumber,"
-            "sub-impl,"
-            "bosphorus,"
-            "louvain-comms,"
-        )
-
         //Occur based simplification
         , perform_occur_based_simp(true)
         , do_strengthen_with_occur       (true)
@@ -375,7 +354,6 @@ DLL_PUBLIC SolverConf::SolverConf() :
         , gatefinder_time_limitM(200)
 
         //Gauss
-        , doM4RI(true)
         , xor_detach_reattach(false)
         , force_use_all_matrixes(false)
 
@@ -396,6 +374,10 @@ DLL_PUBLIC SolverConf::SolverConf() :
         , thread_num(0)
         , is_mpi(false)
 
+        // Oracle
+        , oracle_get_learnts(false) // get oracle learnt clauses
+        , oracle_removed_is_learnt(false) // clauses removed by Oracle should be learnt
+
         //misc
         , origSeed(0)
         , simulate_frat(false)
