@@ -152,7 +152,8 @@ bool SubsumeStrengthen::backw_sub_str_with_long(
     assert(!cl.freed());
 
     if (solver->conf.verbosity >= 6)
-        cout << "backw_sub_str_with_long-ing with clause:" << cl << endl;
+        cout << "backw_sub_str_with_long-ing with clause:" << cl
+            << " offset: " << offset << endl;
 
     find_subsumed_and_strengthened(
         offset
@@ -176,18 +177,13 @@ bool SubsumeStrengthen::backw_sub_str_with_long(
         }
 
         if (subsLits[j] == lit_Undef) {  //Subsume
-            #ifdef VERBOSE_DEBUG
-            if (solver->conf.verbosity >= 6)
-                cout << "subsumed clause " << cl2 << endl;
-            #endif
+            VERBOSE_PRINT("subsumed clause " << cl2);
 
             //If subsumes a irred, and is redundant, make it irred
             if (cl.red()
                 && !cl2.red()
             ) {
-                #ifdef STATS_NEEDED
-                solver->stats_del_cl(&cl);
-                #endif
+                STATS_DO(solver->stats_del_cl(&cl));
                 cl.makeIrred();
                 solver->litStats.redLits -= cl.size();
                 solver->litStats.irredLits += cl.size();
@@ -540,26 +536,10 @@ bool SubsumeStrengthen::handle_added_long_cl(const bool verbose)
     ) {
         const ClOffset offs = simplifier->added_long_cl[i];
         Clause* cl = solver->cl_alloc.ptr(offs);
-        if (cl->freed() || cl->getRemoved())
-            continue;
-
+        if (cl->freed() || cl->getRemoved()) continue;
         cl->stats.marked_clause = 0;
-        if (!backw_sub_str_with_long(offs, stat)) {
-            goto end;
-        }
-
-//         if (cl->size() == 3 &&
-//             !cl->red() &&
-//             !simplifier->perform_ternary(cl, offs, stat))
-//         {
-//             goto end;
-//         }
-
-        if ((i&0xfff) == 0xfff
-            && solver->must_interrupt_asap()
-        ) {
-            goto end;
-        }
+        if (!backw_sub_str_with_long(offs, stat)) goto end;
+        if ((i&0xfff) == 0xfff && solver->must_interrupt_asap()) goto end;
     }
 
     end:
@@ -567,9 +547,7 @@ bool SubsumeStrengthen::handle_added_long_cl(const bool verbose)
     for(; i < simplifier->added_long_cl.size(); i ++) {
         ClOffset off = simplifier->added_long_cl[i];
         Clause* cl = solver->cl_alloc.ptr(off);
-        if (cl->freed() || cl->getRemoved())
-            continue;
-
+        if (cl->freed() || cl->getRemoved()) continue;
         cl->stats.marked_clause = 0;
     }
     simplifier->added_long_cl.clear();
