@@ -23,10 +23,17 @@ THE SOFTWARE.
 #ifndef __VARDATA_H__
 #define __VARDATA_H__
 
+#include <limits>
+#ifdef ARJUN_SERIALIZE
+#include <boost/serialization/split_member.hpp>
+#endif
+
 #include "constants.h"
 #include "propby.h"
 #include "avgcalc.h"
 
+using std::numeric_limits;
+
 namespace CMSat
 {
 
@@ -63,15 +70,74 @@ struct VarData
     uint8_t saved_polarity:1;
     uint8_t best_polarity:1;
     uint8_t inv_polarity:1;
-    bool propagated = false;
     uint8_t is_bva:1;
     uint8_t occ_simp_tried:1;
-
+    bool propagated = false;
 
     #if defined(STATS_NEEDED)
     uint32_t community_num = numeric_limits<uint32_t>::max();
     #endif
 
+    template<class Archive>
+    void save(Archive& ar, const unsigned int /*version*/) const {
+        ar << level;
+        ar << sublevel;
+        #ifdef WEIGHTED_SAMPLING
+        ar << weight;
+        #endif
+        ar << reason;
+        ar << assumption;
+        ar << removed;
+
+        //bitfield
+        const bool my_stable_polarity = stable_polarity;
+        const bool my_saved_polarity  = saved_polarity;
+        const bool my_best_polarity   = best_polarity;
+        const bool my_inv_polarity    = inv_polarity;
+        const bool my_is_bva          = is_bva;
+        const bool my_occ_simp_tried  = occ_simp_tried;
+        ar << my_stable_polarity;
+        ar << my_saved_polarity;
+        ar << my_best_polarity;
+        ar << my_inv_polarity;
+        ar << my_is_bva;
+        ar << my_occ_simp_tried;
+
+        #if defined(STATS_NEEDED)
+        ar << community_num;
+        #endif
+        ar << propagated;
+    }
+
+    template<class Archive>
+    void load(Archive& ar, const unsigned int /*version*/) {
+        ar >> level;
+        ar >> sublevel;
+        #ifdef WEIGHTED_SAMPLING
+        ar >> weight;
+        #endif
+        ar >> reason;
+        ar >> assumption;
+        ar >> removed;
+
+        //bitfield
+        bool bit;
+        ar >> bit; stable_polarity = bit;
+        ar >> bit; saved_polarity = bit;
+        ar >> bit; best_polarity = bit;
+        ar >> bit; inv_polarity = bit;
+        ar >> bit; is_bva = bit;
+        ar >> bit; occ_simp_tried = bit;
+
+        #if defined(STATS_NEEDED)
+        ar >> community_num;
+        #endif
+        ar >> propagated;
+    }
+#ifdef ARJUN_SERIALIZE
+    BOOST_SERIALIZATION_SPLIT_MEMBER()
+#endif
+
     #if defined(STATS_NEEDED_BRANCH) || defined(FINAL_PREDICTOR_BRANCH)
     uint32_t set = 0;
     uint64_t num_propagated = 0;
